# -*- coding: utf-8 -*-
#
# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
#
# v3.0~mickyz

import gtk, gobject, gui, modules, tools

from tools import consts, prefs
from gettext import gettext as _

(
    ROW_ENABLED,    # true if module is enabled
    ROW_TEXT,       # name and description of module
    ROW_ICON,       # prefs icon if module is configurable
    ROW_UNLOADABLE, # true if module can be disabled
    ROW_INSTANCE,   # instance of the module, if any
    ROW_MODINFO     # information exported by module
) = range(6)

class Preferences:

    def __init__(self):
        """ Contructor """
        from gui import window

        self.scrollList = [None, None]
        self.window = window.Window('Preferences.glade', 'vbox1',
            prefs.getWidgetsTree().get_object('win-main'),
            _('Preferences'), 400, 360)
        self.window.getWidget('notebook1').connect('switch-page', self.fillList)
        #self.window.getWidget('btn-help').connect('clicked', self.onHelp) 
        self.window.getWidget('btn-prefs').connect('clicked', self.onPreferences)  
        self.window.getWidget('btn-close').connect('clicked', lambda w:
            self.window.hide())

    def buildList(self, page_num):
        """ Set an empty list for the modules """
        from gui import extListview

        scrollwin = ['scrolledwindow1', 'scrolledwindow2']
        toggleRdr = gtk.CellRendererToggle()
        columns   = (
            ('',   [(toggleRdr, gobject.TYPE_BOOLEAN)],
            ROW_ENABLED, False, True),
            ('',   [(gtk.CellRendererText(), gobject.TYPE_STRING)],
            ROW_TEXT, True,  True),
            ('',   [(gtk.CellRendererPixbuf(), gtk.gdk.Pixbuf)],
            ROW_ICON, False, True),
            (None, [(None, gobject.TYPE_BOOLEAN)],
            ROW_UNLOADABLE, False, False),
            (None, [(None, gobject.TYPE_PYOBJECT)],
            ROW_INSTANCE,   False, False),
            (None, [(None, gobject.TYPE_PYOBJECT)],
            ROW_MODINFO,    False, False)
            )

        self.currList = extListview.ExtListView(columns, sortable=False,
            useMarkup=True, canShowHideColumns=False)
        self.currList.set_headers_visible(False)
        self.currList.addColumnAttribute(0, toggleRdr, 'activatable',
            ROW_UNLOADABLE)
        self.currList.get_selection().connect('changed', self.onSelChanged)

        toggleRdr.connect('toggled', self.onModuleToggled, page_num)

        self.window.getWidget(scrollwin[page_num]).add(self.currList)
        self.scrollList[page_num] = self.currList

    def fillList(self, widget, page, page_num):
        """ Fill the list with modules according to the currently category """
        self.currList = self.scrollList[page_num]
        if not self.currList:
            self.buildList(page_num)

        rows = []
        for (name, data) in modules.getModules():
            instance     = data[modules.MOD_INSTANCE]
            category     = data[modules.MOD_INFO][modules.MODINFO_CATEGORY]
            mandatory    = data[modules.MOD_INFO][modules.MODINFO_MANDATORY]
            configurable = data[modules.MOD_INFO][modules.MODINFO_CONFIGURABLE]

            if category:
                if configurable and instance:
                    icon = gtk.Label().render_icon(gtk.STOCK_PREFERENCES,
                        gtk.ICON_SIZE_MENU)
                else:
                    icon = None

                text = '<b>%s</b>\n<small>%s</small>' % (tools.htmlEscape(_(name)),
                    tools.htmlEscape(data[modules.MOD_INFO][modules.MODINFO_DESC]))

                if category == 1 and page_num == 0:
                    rows.append((instance is not None, text, icon, not mandatory,
                       instance, data[modules.MOD_INFO]))

                if category == 2 and page_num == 1:
                    rows.append((instance is not None, text, icon, not mandatory,
                       instance, data[modules.MOD_INFO]))

        rows.sort(key=lambda row: row[ROW_TEXT])

        self.currList.replaceContent(rows)

    def onModuleToggled(self, renderer, path, page_num):
        """ A module has been enabled/disabled """
        row  = self.currList.getRow(path)
        name = row[ROW_MODINFO][modules.MODINFO_NAME]

        if row[ROW_ENABLED]:
            modules.unload(name)
        else:
            try:
                modules.load(name)
            except modules.LoadException, e:
                gui.errorMsgBox(self.window, 'Loading error:', str(e))

        self.fillList(None, None, page_num)

    def onSelChanged(self, selection):
        """ Decide whether the new selection may be configured """
        sensitive = self.currList.getSelectedRowsCount() == 1 and \
            self.currList.getFirstSelectedRow()[ROW_ICON] is not None

        self.window.getWidget('btn-prefs').set_sensitive(sensitive)

    def onPreferences(self, btn):
        """ Configure the selected module """
        self.currList.getFirstSelectedRow()[ROW_INSTANCE].configure(self.window)

    #ef onHelp(self, btn):
    #   """ Display a small help message box """

    def show(self):
        """ Show the dialog box """
        if not self.window.isVisible():
            self.window.getWidget('btn-close').grab_focus()
            self.window.show_all()

__instance = None

def show():
    """ Show the preferences instance """
    global __instance

    if __instance is None:
        __instance = Preferences()
    __instance.show()
