# -*- coding: utf-8 -*-
#
# Author: Ingelrest François (Francois.Ingelrest@gmail.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#
# v3.0~mickyz

import gobject, gtk, os, sys, threading
from tools import consts, prefs, log

(
    MODINFO_NAME,         # Name of module, must be unique
    MODINFO_L10N,         # Name translated into current locale
    MODINFO_DESC,         # Description of module, translated
    MODINFO_MANDATORY,    # If module cannot be disabled
    MODINFO_CONFIGURABLE, # If module can be configured
    MODINFO_CATEGORY,     # The category of module, None to hide
) = range(6)

(
    MOD_PMODULE,   # The actual Python module object
    MOD_CLASSNAME, # The classname of module
    MOD_INSTANCE,  # Instance, None if not enabled
    MOD_INFO       # A tuple exported by the module
) = range(4)

def load(name):
    """ Load the given module, may raise LoadException """
    mModulesLock.acquire()
    module = mModules[name]
    mModulesLock.release()

    try:
        module[MOD_INSTANCE] = getattr(module[MOD_PMODULE], module[MOD_CLASSNAME])()
        module[MOD_INSTANCE].start()

        mHandlersLock.acquire()
        if module[MOD_INSTANCE] in mHandlers[consts.MSG_EVT_MOD_LOADED]:
            module[MOD_INSTANCE].postMsg(consts.MSG_EVT_MOD_LOADED)
        mHandlersLock.release()

        print 'Loaded: %s' % module[MOD_CLASSNAME]
        mEnabledModules.append(name)
        prefs.set(__name__, 'enabled_modules', mEnabledModules)
    except:
        import traceback
        log.c.error(traceback.format_exc())

def unload(name):
    """ Unload the given module """
    mModulesLock.acquire()
    module               = mModules[name]
    instance             = module[MOD_INSTANCE]
    module[MOD_INSTANCE] = None
    mModulesLock.release()

    if instance is not None:
        mHandlersLock.acquire()
        instance.postMsg(consts.MSG_EVT_MOD_UNLOADED)
        for handlers in [handler for handler in mHandlers.itervalues()
            if instance in handler]:
            handlers.remove(instance)
        mHandlersLock.release()

        mEnabledModules.remove(name)
        print 'Unload: %s' % module[MOD_CLASSNAME]
        prefs.set(__name__, 'enabled_modules', mEnabledModules)

def getModules():
    """ Return a copy of all known modules """
    mModulesLock.acquire()
    copy = mModules.items()
    mModulesLock.release()
    return copy

def register(module, msgList):
    """ Register the module for all messages in the list/tuple """
    mHandlersLock.acquire()
    for msg in msgList:
        mHandlers[msg].add(module)
    mHandlersLock.release()

def showPreferences():
    """ Show the preferences dialog box """
    from gui import preferences
 
    gobject.idle_add(preferences.show)

def __postMsg(msg, params={}):
    """ The postMsg function, which must be executed in the GTK main loop """
    mHandlersLock.acquire()
    for module in mHandlers[msg]:
        module.postMsg(msg, params)
    mHandlersLock.release()

def postMsg(msg, params={}):
    """ Post a message to the queue of modules that registered """
    # we need to ensure that posting messages will be done by the gtk main loop
    # otherwise, the code of threaded modules could be executed in the caller's
    # thread, which could cause problems when calling gtk functions
    gobject.idle_add(__postMsg, msg, params)

def __postQuitMsg():
    """ The postQuitMsg function, which must be executed in the GTK main loop """
    __postMsg(consts.MSG_EVT_APP_QUIT)
    for modData in mModules.itervalues():
        if modData[MOD_INSTANCE] is not None:
            modData[MOD_INSTANCE].join()
    # don't exit the application right now, let modules do their job before
    gobject.idle_add(gtk.main_quit)


def postQuitMsg():
    """ Post a MSG_EVT_APP_QUIT in each module's queue and exit the application """
    # as with postMsg(), to ensure that code will be executed by gtk main loop
    gobject.idle_add(__postQuitMsg)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

class ModuleBase:
    """ This class makes sure that all modules have some mandatory functions """

    def join(self):
        pass
    def start(self):
        pass
    def configure(self, parent):
        pass
    def handleMsg(self, msg, params):
        pass        
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

class Module(ModuleBase):
    """ The base class for non-threaded modules """

    def __init__(self, handlers):
        self.handlers = handlers
        register(self, handlers.keys())

    def postMsg(self, msg, params={}):
        gobject.idle_add(self.__dispatch, msg, params)

    def __dispatch(self, msg, params):
        self.handlers[msg](**params)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

class ThreadedModule(threading.Thread, ModuleBase):
    """ The base class for threaded modules """

    def __init__(self, handlers):
        """ Constructor """
        import Queue

        # attributes
        self.queue        = Queue.Queue(0) # list of queued messages
        self.gtkResult    = None # value returned by function executed
        self.gtkSemaphore = threading.Semaphore(0) # used to execute some code

        # initialization
        threading.Thread.__init__(self)

        # add QUIT and UNLOADED messages if needed
        # these messages are required to exit the thread's loop
        if consts.MSG_EVT_APP_QUIT not in handlers:
            handlers[consts.MSG_EVT_APP_QUIT] = lambda: None
        if consts.MSG_EVT_MOD_UNLOADED not in handlers:
            handlers[consts.MSG_EVT_MOD_UNLOADED] = lambda: None

        self.handlers = handlers
        register(self, handlers.keys())

    def __gtkExecute(self, func):
        """ Private function, must be executed in the GTK main loop """
        self.gtkResult = func()
        self.gtkSemaphore.release()

    def gtkExecute(self, func):
        """
            Execute func in the GTK main loop,
            block the execution of the thread until done
        """
        gobject.idle_add(self.__gtkExecute, func)
        self.gtkSemaphore.acquire()
        return self.gtkResult

    def threadExecute(self, func, *args):
        """
            Schedule func(*args) to be called by the thread
            This is used to avoid func to be executed in the GTK main loop
        """
        self.postMsg(consts.MSG_CMD_THREAD_EXECUTE, (func, args))

    def postMsg(self, msg, params={}):
        """ Enqueue a message in this threads's message queue """
        self.queue.put((msg, params))

    def run(self):
        """ Wait for messages and handle them """
        msg = None
        while msg != consts.MSG_EVT_APP_QUIT and msg != consts.MSG_EVT_MOD_UNLOADED:
            (msg, params) = self.queue.get(True)

            if msg == consts.MSG_CMD_THREAD_EXECUTE:
                (func, args) = params
                func(*args)
            else:
                self.handlers[msg](**params)
#_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

mModDir = os.path.dirname(__file__) # where modules are located
mModules = {} # all known modules associated to an 'active' boolean
mHandlers = dict([(msg, set()) for msg in xrange(consts.MSG_END_VALUE)])
mModulesLock = threading.Lock() # protects the modules list from concurrent access
mHandlersLock = threading.Lock() # protects the handlers list from concurrent access
mEnabledModules = prefs.get(__name__, 'enabled_modules', []) # list of modules enabled

# find modules, instantiate those that are mandatory or previously enabled by user
sys.path.append(mModDir)

for file in [os.path.splitext(file)[0] for file in os.listdir(mModDir)
    if file.endswith('.py') and file != '__init__.py']:
    try:
        pModule = __import__(file)
        modInfo = getattr(pModule, 'MOD_INFO')

        # should it be instanciated?
        instance = None
        if modInfo[MODINFO_MANDATORY] or modInfo[MODINFO_NAME] in mEnabledModules:
            instance = getattr(pModule, file)()
            instance.start()
            print 'Loaded: %s' % file

        # add it to the dictionary
        mModules[modInfo[MODINFO_NAME]] = [pModule, file, instance, modInfo]
    except:
        import traceback
        log.c.error(traceback.format_exc())

# remove enabled modules that are no longer available
mEnabledModules[:] = [module for module in mEnabledModules if module in mModules]
prefs.set(__name__, 'enabled_modules', mEnabledModules)
